        PROGRAM LLCA
C-----------------------------------------------------------------------
C
C  Located latent class analysis
C
C  This program estimates the Located Latent Class Model, a latent
C  class model with latent classes located on a latent trait
C  dimension.  These models are also called discrete latent trait
C  models (Heinen T, Latent class and discrete latent trait models,
C  Sage Publications, 1996).
C
C  The model and estimation algorithm are described in Uebersax JS,
C  Statistical modeling of expert ratings on medical treatment
C  appropriateness, JASA, 1993, 88, 421-427.
C
C  Dichotomous and ordered-category data are supported.  Response
C  functions may be probit or logistic--in the latter case, with
C  dichotomous data, the model is identical to the Rasch-type latent
C  class model described by Lindsay, Clogg and Grego, JASA, 1991,
C  86, 96-107.
C
C  This program uses JP Chandler's STEPIT subroutine for nonlinear
C  optimization (for information, see comments in the subroutine
C  header).
C
C  Program source code and an ASCII text User's Guide are available
C  at StatLib.  Both compiled and source code, and a formatted
C  User's Guide are available at:  http://members.xoom.com/jsuebersax/
C  or from the author at jsuebersax@yahoo.com.
C
C  Permission is given to freely use and distribute this program for
C  noncommercial purposes.  Appropriate citation is requested.
C-----------------------------------------------------------------------
C   Version:    1.0
C   Date:       10/06/93
C   Revised:    Change(s)
C   --------    --------------------------------------------------------
C   10/08/93    Provisional fix on XMAX for prevalence parameters
C   11/08/93    Range for latent class location parameters expanded
C               to +/- 15
C   03/25/94    Modified for Lahey FORTRAN; fixed bug in PHI2 and ERFCC
C   01/15/99    Updated header comments, fixed chisq/df bug
C               Commented out lines to fix compiler warnings
C-----------------------------------------------------------------------
C   to do:   standard errors
C            automatic start values
C            raw data input
C            ELC option
C            prior distribution on alphas
C            update model table
C  
C-----------------------------------------------------------------------
C
C   LIST BUGS HERE:
C
C-----------------------------------------------------------------------
C  For information or comments, contact:
C
C      John S. Uebersax
C      email:  jsuebersax@yahoo.com
C      or:     71302.2362@compuserve.com
C-----------------------------------------------------------------------
C  GLOSSARY
C
C  Routines:
C
C    Name        Type      Purpose
C    -------------------------------------------------------------------
C    LLCA        Main      Read commands; call subroutines
C    OBJFUN      Sub       Function to minimize (LR chisquare)
C    REPORT      Sub       Output results
C    INIT        Sub       Initialize index vectors and STEPIT variables
C    RECRUIT     Sub       Calculate recruitment probabilities
C    DATAIN      Sub       Input observed frequncies
C    STARTV      Sub       Input/Output start values
C    PHI1        Func      Use ERFCC to calculate normal cdf value
C    ERFCC       Func      Complementary error function
C
C  Variables and arrays:
C
C    CHISQL                LR chi square
C    CHISQP                Pearson chi square
C    NCELLS                No. of rating patterns
C    NDIS                  No. of independent dispersion (alpha)
C                            parameters
C    NEST                  No. of estimated parameters
C    NR                    No. of items or raters
C    NT                    No. of threshold parameters per item or rater
C    LC                    No. of latent classes
C    MC                    No. of manifest categories
C    TOTAL                 Sum of observed frequencies (total N)
C
C    A      (        i)    Dispersion parameter for item i
C    B      (        j)    Location parameter for latent class j
C    BIAS   (        i)    Bias parameter for item i (optional)
C    R      (        j)    Prevalence of latent class j
C    P      (  i, j, k)    Probability of response level k given
C                           latent class j for item i
C    T      (     i, k)    Threshold parameter for item i, level k
C
C    FO     (     1000)    Observed pattern frequencies
C    FE     (     1000)    Expected pattern frequencies
C    IOP    (       10)    Run options
C    IVEC   ( 1000, 30)    Response patterns
C
C  Options:
C
C    IOP( 1)  Use previous start values (0=no, 1=yes)
C    IOP( 2)  Indexed frequencies (0=no, 1=yes)
C    IOP( 3)  Equal measurement error (0=no, 1=yes)
C    IOP( 4)  Ogive function (0=normal, 1=logistic)
C    IOP( 6)  Constrained thresholds (1=simple bias, 2=equal thresholds)
C    IOP( 8)  Print expected frequencies (0=no, 1=yes)
C    IOP( 9)  Print conditional rating probabilities (0=no, 1=yes)
C    IOP(10)  Print classification information (0=no, 1=yes)
C
C-----------------------------------------------------------------------

        IMPLICIT DOUBLE PRECISION(A-H, O-Z)
        COMMON /FASTER/ JVARY
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

        DIMENSION       NF(10)
C       DATA BLNK /4H    /

        OPEN (UNIT=5, FILE='INPUT' )
        OPEN (UNIT=1, FILE='DATA'  )
        OPEN (UNIT=2, FILE='OUTPUT')

        WRITE (6, 6300)
        READ (5, 5000) TITLE
        READ (5, *)    IMAX
C
C  Use previous start values?
C
        READ (5, 5001) IOP(1)

        READ (5, 5001) LC
        READ (5, 5001) MC
        READ (5, 5001) NR
C
C  Indexed frequencies?
C
        READ (5, 5001) IOP(2)
C
C  Unequal dispersions?
C
        READ (5, 5001) IOP(3)
        NDIS = NR
        IF (IOP(3) .EQ. 1) NDIS = 1
C
C  Ogive type
C
        READ (5, 5001) IOP(4)
C
C  Constrained thresholds?
C
        READ (5, 5001) IOP(6)
        NT = NR
        IF (IOP(6) .GT. 0) NT = 1
        IF (IOP(6) .NE. 2) THEN
          DO 2 I = 1, NR
          BIAS(I) = 0.
    2     CONTINUE
        ENDIF
C
C  Print expected frequencies?
C
        READ (5, 5001) IOP(8)
C
C  Option to print conditional probabilities
C
    5   READ (5, 5001) IOP(9)
C
C  Assign cases?
C
        READ (5, 5001) IOP(10)
C
C  Parameters to fix
C
        NFIXED  =   0
        DO 10 I = 1, 10
        READ (5, 5001) NF(I)
        IF (NF(I) .NE. 0) NFIXED = NFIXED + 1
   10   CONTINUE
C
C  Input start values into transfer vector
C
        IF (IOP(1) .EQ. 1) GOTO 80
C
C       latent class locations
C
   25   INDX = 0
        DO 30 I = 1, LC
        INDX = INDX + 1
        READ (5, 5002) X(INDX)
   30   CONTINUE
C
C       input latent class prevalences and scale them to sum to 1
C
        XXSUM = 0.0
        ISAVE = INDX
        DO 35 I = 1, LC
        INDX = INDX + 1
        READ (5, 5002) X(INDX)
        XXSUM = XXSUM + X(INDX)
   35   CONTINUE
        DO 38 I = ISAVE + 1, INDX
        X(I) = X(I) / XXSUM
   38   CONTINUE
C
C       measurement (alpha) error parameters
C
        DO 45 I = 1, NDIS
        INDX = INDX + 1
        READ (5, 5002) X(INDX)
   45   CONTINUE
C
C       bias parameters
C
        IF (IOP(6) .NE. 1) GOTO 55
        DO 50 I = 1, NR - 1
        INDX = INDX + 1
        READ (5, 5002) X(INDX)
   50   CONTINUE
C
C       threshold parameters
C
   55   DO 60 I = 1, NT
        DO 60 J = 2, MC
        INDX = INDX + 1
        READ (5, 5002) X(INDX)
   60   CONTINUE
C
C  Input observed data
C
   80   CALL DATAIN
C
C  Initialize things for STEPIT and index vectors
C
        CALL INIT (NFIXED)
        IF (IOP(1) .NE. 1) GOTO 100
C
C  Read starting values from START
C
   90   CALL STARTV (0)
C
C  Fix some parameters
C
  100   DO 220 I = 1, 10
        IF (NF(I) .GT. 0) MASK( NF(I) ) = 1
  220   CONTINUE
C
C  Constrain latent class locations to +/- 15 range      11/07/93
C  (previously +/- 10.0)                                 11/07/93
C
        DO 230 I = 1, LC
        XMIN(I) = -15.0000
        XMAX(I) =  15.0000
  230   CONTINUE
C
C  Constrain latent class probabilities to 0-1 range
C

C****NOTE:  May be better way to handle prevalence parameters
C           and speed up estimation

        DO 240 I = LC + 1, 2 * LC
        XMIN(I) =  0.0000
C
C**Apparently for XMIN to work XMAX must have a value    10/08/93
C**so make it real big                                   10/08/93
C       XMAX(I) =  1.0000
        XMAX(I) =  1.0E8

  240   CONTINUE
C
C  Constrain alpha parameters to be positive
C
        DO 250 I = 2 * LC + 1, 2 * LC + NDIS
        XMIN(I)  =   .01
        XMAX(I)  = 10.00
  250   CONTINUE
C
C  Call optimization routine
C
C       CALL STEPIT(OBJFUN)   (some compilers don't like this)
        CALL STEPIT
C
C  Convergence reached
C
        WRITE (6,6310)
C
C Write OUTPUT
C
C       CALL RESCALE
        CALL REPORT (1)
C
C  Write starting values
C
        CALL STARTV(1)
 5000   FORMAT(20A4       )
 5001   FORMAT(I4         )
 5002   FORMAT(D10.2      )
 6300   FORMAT(1X, 'BEGIN')
 6310   FORMAT(/, 1X, 'Convergence reached')
        END

        SUBROUTINE OBJFUN
C-----------------------------------------------------------------------
C SUBROUTINE OBJFUN -- Objective function to minimize/maximize, in this
C   case, minimize likelihood ratio chi-square, which is the same as
C   maximizing the log-likelihood
C                                                                      
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   04/24/92  Improvements to code
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)
        COMMON /FASTER/ JVARY
        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

        INDX =  0
        DO 12 I = 1, LC
        INDX =  INDX + 1
        B(I) =  X(INDX)
   12   CONTINUE
        RTOT =  0.0
        DO 14 I = 1, LC
        INDX =  INDX + 1
        R(I) =  X(INDX)
        RTOT =  RTOT + R(I)
   14   CONTINUE
        DO 15 I = 1, LC
        R(I) =  R(I)/RTOT
   15   CONTINUE
        DO 16 I = 1, NDIS
        INDX = INDX + 1
        A(I) = X(INDX)
   16   CONTINUE
        IF (IOP(6) .NE. 1) GOTO 18
        SUM  = 0.
        DO 17 I = 1, NR - 1
        INDX =  INDX + 1
        BIAS(I) = X(INDX)
        SUM  = SUM + BIAS(I)
   17   CONTINUE
        BIAS(NR) = 0. - SUM
   18   DO 19 I = 1, NT
        DO 19 J = 2, MC
        INDX = INDX + 1
        T(I, J) = X(INDX)
   19   CONTINUE
C
C  Calculate conditional probabilities
C
        DO 30 I = 1, NR
        DO 30 J = 1, LC
        AA = A(1)
        IF (IOP(3) .EQ. 0) AA = A(I)
        II = 1
        IF (IOP(6) .EQ. 0) II = I
        OFS = 0.0
        IF (IOP(6) .EQ. 1) OFS = BIAS(I)
C
C       manifest category 1
C
        SD = 1. / AA
        Z  = (T(II, 2) + OFS - B(J)) / SD
        IF (IOP(4) .EQ. 0) THEN
          P(I, J, 1) = PHI1(Z)
        ELSE
        P(I, J, 1) =
     .    1 / (1 + DEXP(-1.7 * AA * ((T(II, 2) + OFS) - B(J))))
        END IF
        SUM = P(I, J, 1)
C
C       manifest categories 2 to (MC - 1)
C
        DO 25 K = 2, MC - 1
        Z1  = (T(II, K  ) + OFS - B(J)) / SD
        Z2  = (T(II, K+1) + OFS - B(J)) / SD
        IF (IOP(4) .EQ. 0) THEN
          P(I, J, K) = PHI1(Z2) - PHI1(Z1)
        ELSE
        P(I, J, K) =
     .    1 / (1 + DEXP(-1.7 * AA * ((T(II, K+1) + OFS) - B(J)))) -
     .   (1 / (1 + DEXP(-1.7 * AA * ((T(II, K  ) + OFS) - B(J)))))
        ENDIF
        SUM = SUM + P(I, J, K)
   25   CONTINUE
C
C       manifest category MC
C
        P( I, J, MC) = 1 - SUM

   30   CONTINUE
C
C  Expected frequencies
C
        ETOTAL = 0.0
        DO 40 I = 1, NCELLS
        IF (IOP(2) .EQ. 1 .AND. FO(I) .EQ. 0.0) GOTO 40
        SUM = 0.
        DO 35 J = 1, LC
        PROD = R(J)
        DO 32 K = 1, NR
        PROD = PROD * P(K, J, IVEC(I, K))
   32   CONTINUE
        SUM = SUM + PROD
   35   CONTINUE
        FE(I) = SUM * TOTAL
        ETOTAL = ETOTAL + FE(I)
   40   CONTINUE
C
C  Evaluate fit
C
        CHISQL = 0.0
        DO 50 II = 1, NCELLS
        IF (FE(II) .LT. 1.D-10) FE(II) = 1.D-10
        IF (FO(II) .EQ. 0.0)    GOTO 45
        CHISQL = CHISQL - 2.0 * FO(II) * DLOG(FE(II)/FO(II))
   45   CONTINUE
   50   CONTINUE
        CHISQ = CHISQL

        ICALLS = ICALLS + 1
        WRITE (6,6000) ICALLS, CHISQ
        IF (ICALLS .LT. IMAX) GOTO 200
C
C  Maximum number of iterations reached without convergence
C
        WRITE (6, 6010)
C
C  Write start values
C
        CALL STARTV (1)
C
C  Write OUTPUT
C
        CALL REPORT (0)
        STOP
  200   CONTINUE
        RETURN
 6000   FORMAT(1X, I5, 5X, F16.8)
 6010   FORMAT(/,' Convergence not reached')
        END

        SUBROUTINE REPORT (ICODE)
C-----------------------------------------------------------------------
C SUBROUTINE REPORT -- OUTPUT RESULTS
C
C   ICODE  =  0    convergence not reached
C             1    convergence reached
C-----------------------------------------------------------------------
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   04/23/92  Made separate subroutine
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)
        COMMON /ICALL/  ICALLS, IMAX

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

        DIMENSION       RHO(30)

C
C  Problem information
C
        WRITE (2, 2000) TITLE
        WRITE (2, 2010)
        WRITE (2, 2010)
        WRITE (2, 2020)
        WRITE (2, 2010)
        WRITE (2, 2030) NR
        WRITE (2, 2040) LC
        WRITE (2, 2050) MC
        WRITE (2, 2010)
        IF (IOP(3) .EQ. 1) WRITE (2, 2060)
        IF (IOP(4) .NE. 1) WRITE (2, 2062)
        IF (IOP(4) .EQ. 1) WRITE (2, 2066)
        IF (IOP(6) .EQ. 1) WRITE (2, 2070)
        IF (IOP(6) .EQ. 2) WRITE (2, 2075)
C
C  Fit statistics
C
        WRITE (2, 2010)
        WRITE (2, 2010)
        WRITE (2, 2080)
        WRITE (2, 2010)
        WRITE (2, 2090) CHISQL, CHISQL
        XLOGL    = 0.
        CHISQP   = 0.
        SUME     = 0.
        DO 5  I  = 1, NCELLS
        XLOGL    = XLOGL + FO(I) * DLOG(FE(I) / TOTAL)
        CHISQP   = CHISQP + ((FO(I) - FE(I))**2) / FE(I)
        SUME     = SUME + FE(I)
    5   CONTINUE
        CHISQP   = CHISQP + TOTAL - SUME
        AIC      = -2. * XLOGL + 2. * FLOAT(NEST)
        SCHWARZ  = -2. * XLOGL + DLOG(TOTAL) * FLOAT(NEST)
        NDF      = (MC**NR) - 1 - NEST
        WRITE (2, 2100) CHISQP, CHISQP
        WRITE (2, 2110) NEST
        WRITE (2, 2120) NDF
        WRITE (2, 2010)
C
C  Temporary bug fix:  suppress chisq/df print if df = 0
C
        IF (NDF .GT. 0) WRITE (2, 2130) CHISQP/NDF

        WRITE (2, 2140) XLOGL, XLOGL
        WRITE (2, 2150) AIC, AIC
        WRITE (2, 2160) SCHWARZ, SCHWARZ
        WRITE (2, 2010)
        WRITE (2, 2170) ICALLS
        IF (ICODE .EQ. 1) WRITE (2, 2172)
        IF (ICODE .EQ. 0) WRITE (2, 2174)
C
C  Parameter estimates
C
        WRITE (2, 2010)
        WRITE (2, 2010)
        WRITE (2, 2180)
        WRITE (2, 2010)
        WRITE (2, 2190)
C
C  Latent class locations and probabilities
C
        WRITE (2, 2010)
        DO 10 I = 1, LC
        WRITE (2, 2200) I, B(I)
   10   CONTINUE
        WRITE (2, 2010)
        WRITE (2, 2210)
        WRITE (2, 2010)
        DO 20 I = 1, LC
        WRITE (2, 2200) I, R(I)
   20   CONTINUE
C
C  Latent correlations
C
C
C       calculate latent trait mean
C
        XX = 0.
        DO 30 I = 1, LC
        XX = XX + R(I) * B(I)
   30   CONTINUE
C
C       calculate latent trait variance
C
        YY = 0.
        DO 40 I = 1, LC
        YY = YY + R(I) * (B(I) - XX) ** 2
   40   CONTINUE
C
C       calculate correlations
C
        VART = YY
        XSUM1 = 0.
        XSUM2 = 0.
        DO 45 I = 1, NDIS
        VARE = 1 / (A(I) ** 2)
        RHOSQ = VART / (VART + VARE)
        RHO(I) = RHOSQ ** .5
        XSUM1 = XSUM1 + A(I)   / NDIS
        XSUM2 = XSUM2 + RHO(I) / NDIS
   45   CONTINUE
        WRITE (2, 2010)
        WRITE (2, 2220)
        DO 50 I = 1, NDIS
        IF (NDIS .GT. 1) WRITE (2, 2230) I, A(I), RHO(I)
        IF (NDIS .EQ. 1) WRITE (2, 2232) A(I), RHO(I)
   50   CONTINUE
        IF (NDIS .GT. 1) WRITE (2, 2240) XSUM1, XSUM2
C
C  Threshold estimates
C
   70   WRITE (2, 2010)
        WRITE (2, 2250)
        DO 80 I = 1, NT
C       DO 80 I = 1, NR
        IF (I .GT. 1 .AND. MC .GT. 2) WRITE (2, 2010)
        DO 80 J = 2, MC
C       WRITE (2, 3000) T(1, J) + BIAS(I)
        IF (IOP(6) .EQ. 0) WRITE (2, 2260) I, J, T(I, J)
        IF (IOP(6) .NE. 0) WRITE (2, 2262) J, T(I, J)
   80   CONTINUE
C       IF (MC .GT. 2) WRITE (2, 2010)
C       IF (MC .GT. 2) WRITE (2, 2270)

C
C  Bias estimates
C
        IF (IOP(6) .NE. 1) GOTO 90
        WRITE (2, 2010)
        DO 85 I = 1, NR
        WRITE (2, 3000) I, BIAS(I)
   85   CONTINUE
   90   IF (IOP(8) .EQ. 0) GOTO 112
        WRITE (2, 2010)
        WRITE (2, 2010)
        WRITE (2, 2280)
        WRITE (2, 2010)
        WRITE (2, 2290)
C
C  Observed and expected frequencies
C
        SUME = 0.0

        DO 110 I = 1, NCELLS
          IF (NR .LE. 15) THEN
            WRITE (2, 2300) I, FO(I), FE(I), (IVEC(I, J), J = 1, NR)
          ELSE
            WRITE (2, 2302) I, FO(I), FE(I), (IVEC(I, J), J = 1, NR)
          END IF
        SUME = SUME + FE(I)
  110   CONTINUE

        WRITE (2, 2010)
        WRITE (2, 2310) TOTAL
        WRITE (2, 2320) SUME
C
C  Conditional rating probabilities
C
  112   IF (IOP(9) .EQ. 0) GOTO 122
        WRITE (2, 2010)
        WRITE (2, 2010)
        WRITE (2, 2325)
        DO 120 I = 1, NR
        IF (I. GT. 1) WRITE (2, 2010)
        DO 120 J = 1, LC
        DO 120 K = 1, MC
        WRITE (2, 2330)  I, J, K, P(I, J, K)
  120   CONTINUE
C
C  Latent class assignments and latent trait scores
C
  122   IF (IOP(10) .EQ. 0) RETURN
        WRITE (2, 2010)
        WRITE (2, 2010)

        IF (NR .LE. 20) THEN
           WRITE (2, 2340)
        ELSE
           WRITE (2, 2342)
        END IF
        CALL RECRUIT

 2000   FORMAT ( 'LLCA -- Located Latent Class Analysis', /,
     .           'Version 1.0', //, 20A4)
 2010   FORMAT ( '')
 2020   FORMAT ( '1. DESIGN INFORMATION' )
 2030   FORMAT ( '   Number of items or raters:       ', I6 )
 2040   FORMAT ( '   Number of latent classes:        ', I6 )
 2050   FORMAT ( '   Number of manifest categories:   ', I6 )
 2060   FORMAT ( '   Equal measurement error restriction' )
 2062   FORMAT ( '   Normal ogive' )
 2066   FORMAT ( '   Logistic ogive' )
 2070   FORMAT ( '   Simple bias model' )
 2075   FORMAT ( '   Identical thresholds model' )
 2080   FORMAT ( '2. FIT STATISTICS'  )
 2090   FORMAT ( '   Likelihood ratio chi-square      ', F9.2, 4X,F14.6)
 2100   FORMAT ( '   Pearson chi-square               ', F9.2, 4X,F14.6)
 2110   FORMAT ( '   Number of estimated parameters   ', I6   )
 2120   FORMAT ( '   Degrees of freedom               ', I6   )
 2130   FORMAT ( '   Pearson chi-square / df          ', F9.2 )
 2140   FORMAT ( '   Ln-likelihood                    ', F9.2, 4X,F14.6)
 2150   FORMAT ( '   AIC                              ', F9.2, 4X,F14.6)
 2160   FORMAT ( '   Schwarz index                    ', F9.2, 4X,F14.6)
 2170   FORMAT ( '   STEPIT iterations                ', I6   )
 2172   FORMAT ( '   Convergence reached')
 2174   FORMAT ( '   Convergence not reached')
 2180   FORMAT ( '3. PARAMETER ESTIMATES'                          )
 2190   FORMAT ( '   a. Latent class location parameters'   )
 2200   FORMAT ( I7, F10.3, 7X, 6A4 )
 2210   FORMAT ( '   b. Latent class probabilities  ' )
 2220   FORMAT ( '   c. Measurement error'
     .   //      '      Item/                       '
     .   /       '      rater      alpha      rho   '
     .   /       '      ------     -----     -----  ' )
 2230   FORMAT ( I10, 2X, 2F10.3)
 2232   FORMAT ( 7X, 'all',  2X, 2F10.3)
 2240   FORMAT ( 12X, '     -----     -----'
     .   /       6X, 'Mean', 2X, 2F10.3)
 2250   FORMAT ( '   d. Threshold parameters'
     .   //      '      Item/     Manifest    Threshold'
     .   /       '      rater     category     estimate'
     .   /       '      ------    --------    ---------' )
 2260   FORMAT ( I10, I11, F15.3 )
 2262   FORMAT ( 7X, 'all', 2X, I10, F15.3 )
C2270   FORMAT ( 7X, 'Mean' )
 2280   FORMAT ( '4. OBSERVED/EXPECTED FREQUENCIES' )
 2290   FORMAT ( '              Observed       Expected'
     .   /  '   Cell      frequency      frequency', 7X,'Rating Pattern'
     .   /  '   -----     ---------      ---------', 7X, 30(1H-) )
 2300   FORMAT ( I7, F14.0, F15.2, 8X, 15I2)
 2302   FORMAT ( I7, F14.0, F15.2, 8X, 30I1)
 2310   FORMAT ( '   Total number of cases           ', F12.0 )
 2320   FORMAT ( '   Total expected frequencies      ', F16.4 )
 2325   FORMAT ( '5. CONDITIONAL RATING PROBABILITIES'
     .   //      '    Item/   Latent   Manifest   Conditional '
     .   /       '    rater    class   category   probability '
     .   /       '    -----   ------   --------   ----------- ')
 2330   FORMAT ( I8, 2I9, F15.4 )
 2340   FORMAT ( '6.  CLASSIFICATION/SCORING '
     .   //  37X, ' Modal      Latent'
     .   /   12X, ' Observed    Assigned    proba-       trait    '
     .   /   '    Cell    frequency       class    bility       score',
     .       '    Rating Pattern',
     .   /   '    ----    ---------    --------    ------     -------',
     .       '    --------------------' )
 2342   FORMAT ( '6.  CLASSIFICATION/SCORING '
     .   //  37X, ' Modal      Latent'
     .   /   12X, ' Observed    Assigned    proba-       trait    '
     .   /   '    Cell    frequency       class    bility       score',
     .       '    Rating Pattern',
     .   /   '    ----    ---------    --------    ------     -------',
     .       '    ------------------------------' )
 3000   FORMAT ( '      Bias for item/rater', I3, ': ', F7.3 )
        RETURN
        END

        SUBROUTINE INIT (NFIXED)
C-----------------------------------------------------------------------
C INITIALIZE THINGS FOR STEPIT AND INDEX VECTORS
C
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   04/23/92  Made separate subroutine
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

C
C  Generate index vectors for n-way table
C
        IF (IOP(2) .EQ. 1) GOTO 50
        DO 20 I = 1, NR
        II = NR + 1 - I
        INDX = 0
        DO 15 J = 1, MC**(I-1)
        DO 15 K = 1, MC
        IF (II .EQ. 1) LL = 1
        IF (II .GT. 1) LL = MC**(II - 1)
        DO 15 L = 1, LL
        INDX = INDX + 1
        IVEC(INDX, I) = K
   15   CONTINUE
   20   CONTINUE
C
C  Number of estimated parameters
C
   50   NV = NT*(MC - 1) + (2 * LC) + NDIS
        IF (IOP(6) .EQ. 1) NV = NV + (NR - 1)
        NEST = NV - NFIXED
C
C  See STEPIT documentation
C
C       ICALLS     = 0
        NTRACE     = 0
        MATRIX     = 105
        DO 60 I = 1, NV
        XMAX   (I) = 0.0
        XMIN   (I) = 0.0
        DELMIN (I) = 0.0
        DELTAX (I) = X(I) * (.01)
        MASK   (I) = 0
   60   CONTINUE
        RETURN
        END

        SUBROUTINE RECRUIT
C-----------------------------------------------------------------------
C SUBROUTINE RECRUIT -- CALCULATE RECRUITMENT PROBABILITIES           
C                                                                      
C   Revised:  Change(s)                                                
C   --------  ----------------------------------------------------------
C   04/25/92  Version for LLCAA
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

C       DIMENSION CJP(9), UJP(9), PCP(9)
        DIMENSION CJP(9), PCP(9)
        DIMENSION PCT(9), C(9, 9)
        DIMENSION ICOL(8)

C
C  CJP() = Conditional joint probabilities
C  UJP() = Unconditional joint probabilities
C  PCP() = Posterior class membership probabilities
C  PCT() = Percent observed cases assigned to each latent class
C  C  () = True/assigned latent class crossclassification table
C
C
C  Initialize arrays etc.
C
        CORRECT  =  0.D0
        DO 5 I   =  1, LC
        PCT(I)   =  0.D0
        DO 5 J   =  1, LC
        C(I, J)  =  0.D0
    5   CONTINUE
        SUM      =  0.D0
C
C  Make latent class probabilities sum to 1
C
        DO 10 I = 1, LC
        SUM = SUM + R(I)
   10   CONTINUE
        DO 20 I = 1, LC
        R(I) = R(I) / SUM
   20   CONTINUE
C
C  Conditional joint probabilities
C
        DO 250 I = 1, NCELLS
        DO 100 J = 1, LC
        PROD = 1.0D0
        DO  50 K = 1, NR
        INDX = IVEC(I, K)
        PROD = PROD * P(K, J, INDX)
   50   CONTINUE
        CJP(J) = PROD
  100   CONTINUE
C
C  Unconditional joint and posterior class probabilities
C
        DO 200 J = 1, LC
C       UJP(J) = R(J) * CJP(J)
        PCP(J) = R(J) * CJP(J) / (FE(I) / TOTAL)
  200   CONTINUE
        JJ = 0
        PMAX = 0.0D0
        DO 210 J = 1, LC
        IF (PCP(J) .GT. PMAX) JJ = J
        IF (PCP(J) .GT. PMAX) PMAX = PCP(J)
  210   CONTINUE
C
C  Classification table
C
        DO 215 J = 1, LC
        C(J, JJ) = C(J, JJ) + (FO(I)/TOTAL) * PCP(J)
  215   CONTINUE
C
C  Latent trait scores
C
        SCORE = 0.
        DO 220 J = 1, LC
        SCORE = SCORE + B(J) * PCP(J)
  220   CONTINUE
C
C  Percent assigned to each latent class
C
        CORRECT = CORRECT + (FO(I)/TOTAL * PMAX)
        DO 230 J = 1, LC
        IF (JJ .EQ. J) PCT(J) = PCT(J) + FO(I)/TOTAL
  230   CONTINUE
C
C  Print cell #, observed frequency, modal latent class, probability
C  of membership in modal latent class, latent class score, and
C  rating pattern
C
        IF (NR .LE. 10) THEN
          WRITE (2, 2000) I, FO(I), JJ, PMAX, SCORE, (IVEC(I, J), J =
     .     1, NR)
        ELSE
          WRITE (2, 2002) I, FO(I), JJ, PMAX, SCORE, (IVEC(I, J), J =
     .     1, NR)
        END IF

  250   CONTINUE
C
C  Print information on correct and incorrect classifications
C
        WRITE (2, 2010) CORRECT
        DO 260 I = 1, LC
        WRITE (2, 2020) I, PCT(I)
        ICOL(I) = I
  260   CONTINUE
        WRITE (2, 2030)  (ICOL(I), I = 1, LC)
        WRITE (2, 2035)  
        DO 270 I = 1, LC
        WRITE (2, 2040) I, (C(I, J), J = 1, LC)
  270   CONTINUE

 2000   FORMAT( I8, F12.0, I10, F13.4, F12.4, 4X, 15I2 )
 2002   FORMAT( I8, F12.0, I10, F13.4, F12.4, 4X, 30I1 )
 2010   FORMAT( / '  Percent correctly classified:', F8.4,
     .   //       '  Percent assigned to:', / )
 2020   FORMAT(   '    Latent Class ', I2 ,'            ', F8.4   )
 2030   FORMAT( / '  Classification/membership joint probabilities:'
     .   //       '  Latent    Assigned Latent Class'
     .   /        '  Class', 8I8)
 2035   FORMAT( 2X, 6H------, 3X, 62(1H-) )
 2040   FORMAT( I5, 4X, 10F8.4 )
        RETURN
        END

        SUBROUTINE DATAIN
C-----------------------------------------------------------------------
C SUBROUTINE DATAIN -- INPUT DATA
C
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   04/24/92  Made separate subroutine
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

        TOTAL  = 0.0
        NCELLS = MC**NR
        IF (IOP(2) .EQ. 1) GOTO 10
        READ (1, *) (FO(I), I = 1, NCELLS)
        GOTO 30
   10   READ (1, *) NCELLS
        DO 20 I = 1, NCELLS
        READ (1, *) (IVEC(I, J), J = 1, NR), FO(I)
   20   CONTINUE
   30   TOTAL = 0.0
        DO 40 I = 1, NCELLS
        TOTAL = TOTAL + FO(I)
   40   CONTINUE
        RETURN
        END

        SUBROUTINE STARTV (JOP)
C-----------------------------------------------------------------------
C SUBROUTINE STARTV -- INPUT/OUTPUT START VALUES
C
C   Parameter   Value    Explanation
C   ---------   ------   ---------------------------------
C   JOP           0      Read start values to file START
C                 1      Write start values to file START
C
C   Revised:  Change(s)
C   --------  ----------------------------------------------------------
C   04/24/92  Made separate subroutine
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION(A-H, O-Z)

        COMMON /ICALL/  ICALLS,IMAX

        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

        COMMON /BLOCK1/ TITLE  ( 18) ,  VARLBL(30, 6),  IOP  (     10) ,
     .                  FO   ( 1000) ,  FE    ( 1000),  IVEC(1000, 30) ,
     .                  B    (    9) ,  R     (    9),  A    (     30) ,
     .                  T    (30, 9),   BIAS  (   30),  P (30,  9,  9) ,
     .                  CHISQL,   CHISQP,   LC,       MC ,      NCELLS ,
     .                  NDIS,     NEST,     NR,       NT,       TOTAL

        OPEN (UNIT=3, FILE='START')

        IF (JOP .EQ. 1) GOTO 50
C
C  Read start values
C
        DO 10 I = 1, NV
        READ (3, *) X(I)
   10   CONTINUE
C
C  Scale latent class probabilities to sum to 1
C
   15   XXSUM = 0.
        DO 20 I = LC + 1, 2 * LC
        XXSUM = XXSUM + X(I)
   20   CONTINUE
        DO 25 I = LC + 1, 2 * LC
        X(I) = X(I) / XXSUM
   25   CONTINUE
        CLOSE (3)
        RETURN
C
C  Write start values
C
C
C  Scale latent class probabilities to sum to 1
C
   50   XXSUM = 0.
        DO 60 I = LC + 1, 2 * LC
        XXSUM = XXSUM + X(I)
   60   CONTINUE
        DO 70 I = LC + 1, 2 * LC
        X(I) = X(I) / XXSUM
   70   CONTINUE
        DO 80 I = 1, NV
        WRITE (3, 3000) X(I)
   80   CONTINUE
        CLOSE (3)
        RETURN
 3000   FORMAT(F22.17)
        END

        DOUBLE PRECISION FUNCTION PHI1(Z)
C--6/17/92--------------------------------------------------------------
C  Use ERFCC to calculate normal cdf value
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
        PHI1=ERFCC(-Z/DSQRT(2.))/2.
        RETURN
        END

        DOUBLE PRECISION FUNCTION ERFCC(X)
C-----------------------------------------------------------------------
C  Returns the complementary error function erfc(x) with fractional
C  error less that 1.2E-7
C
C  W. H. Press et al.,  Numerical Recipes (FORTRAN version).
C  Cambridge:  Cambridge University Press, 1989; p. 164
C-----------------------------------------------------------------------
        IMPLICIT DOUBLE PRECISION (A-H, O-Z)
C
C Comment -- may need underflow logic
C
        Z=DABS(X)
           T=1./(1.+0.5*Z)
            ERFCC=T*DEXP(-Z*Z-1.26551223+T*(1.00002368+T*(.37409196+
     *        T*(.09678418+T*(-.18628806+T*(.27886807+T*(-1.13520398+
     *        T*(1.48851587+T*(-.82215223+T*.17087277)))))))))
        IF (X.LT.0.) ERFCC=2.-ERFCC
        RETURN
        END


        SUBROUTINE STEPIT
c=======================================================================
c  SUBROUTINE STEPIT -- GENERALIZED MAXIMIZATION/MINIMIATION ROUTINE
c
c  Author:          J. P. Chandler
c                   Physics Department, Indiana University
c  Version:         5.2
c  Written:         August, 1967
c=======================================================================
c  Comments by original Author:
c
c  Available from Quantum Chemistry Program Exchange, I. U. Chemistry
c  Department, Bloomington, Indiana
c
c  (Modififed for FORTRAN G OS/360 MVT)
c=======================================================================
c  Comments by J. Uebersax (January 1999)
c
c  J. P. Chandler has granted free noncommercial use of STEPIT, but
c  requests appropriate citation.  The article that describes the
c  subroutine and its use is:  Chandler JP, 1969.  STEPIT--Finds local
c  minima of a smooth function of several parameters.  Computer program
c  abstract.  Behavioral Science, 14, 81-82.
c
c  A more recent, version of STEPIT is (was?) available by ftp.
c  Contact information for J. P. Chandler:
c
c           Prof. John Chandler
c           Computer Science Department
c           Oklahoma State University
c           Stillwater, OK  74078
c           Phone:  (405) 744-5668
c           Email:  jpc@a.cs.okstate.edu
c
c  Everything here in lowercase is a modification I have made.
c
c  Revisions by John Uebersax
c  Date         Change(s)
c  --------     --------------------------------------------------------
c  10/07/89     Modification for RM FORTRAN
c  02/10/94     Modification for Lahey f77L FORTRAN
c  03/14/96     Modified for Lahey f77P FORTRAN
c  03/26/96     Minor format changes
c-----------------------------------------------------------------------
c     IMPLICIT REAL*8 (A-H,O-Z)                   
      implicit double precision (A-H, O-Z) 

c     COMMON/STPIT/CHISQ,X(70),XMAX(70),XMIN(70),DELTAX(70),ERR(70,70),D
c    1ELMIN(70),MASK(70),NV,NTRACE,MATRIX

c
c these arrays allow 70 estimated parameters
c
        COMMON /STPIT / CHISQ        ,  X      ( 70) ,  XMAX (     70) ,
     .                  XMIN   ( 70) ,  DELTAX ( 70) ,  ERR  ( 70, 70) ,
     .                  DELMIN ( 70) ,  MASK   ( 70) ,  NV             ,
     .                  NTRACE       ,  MATRIX

      COMMON /FASTER/ JVARY

      COMMON/ICALL/ICALLS,IMAX

c     DIMENSION VEC(70),TRIAL(70),XSAVE(70),CHI(70),DX(70),SECOND(2,2)
c     DIMENSION OLDVEC(70),SALVO(70),XOSC(70,15),CHIOSC(15),NFLAT(70)

c
c these arrays allow 70 estimated parameters
c
      dimension VEC(70),TRIAL(70),XSAVE(70),CHI(70),DX(70),SECOND(2,2)
      dimension OLDVEC(70),SALVO(70),XOSC(70,15),CHIOSC(15),NFLAT(70)

c
c     NVMAX=50                                        
c     KW=61                                           
      MOSQUE=15
      RATIO=10.0
      COLIN=0.99
      NCOMP=5
      ACK=2.0
c     SIGNIF =  2.E8                                  
c     HUGE   =  1.E65                                 
      signif =  2.E6                                      
      huge   =  1.D65                                     
C
      JVARY=0
   40 IF(NV)290,290,50
   50 NACTIV=0
      DO 150 I=1,NV
      IF(MASK(I))150,60,150
   60 IF(SIGNIF*DABS(DELTAX(I))-DABS(X(I))) 70,70,100
   70 IF(X(I))90,80,90
   80 DELTAX(I)=0.01
      GO TO 100
   90 DELTAX(I)=0.01*X(I)
  100 IF(DELMIN(I))120,110,120
  110 DELMIN(I)=DELTAX(I)/SIGNIF
  120 IF(XMAX(I)-XMIN(I))130,130,140
  130 XMAX(I)=HUGE
      XMIN(I)=-HUGE
  140 NACTIV=NACTIV+1
      VACD=DMIN1(XMAX(I),X(I))
      X(I)=DMAX1(XMIN(I),VACD)
  150 CONTINUE
      COMPAR=0.0
      IF(NACTIV-1)160,190,180
  160 DO 170 J=1,NV
  170 MASK(J)=0
      GO TO 50
  180 A=NACTIV
      SUB=2.0/(A-1.0)
      P=2.0*(1.0/DSQRT(A)/(1.0-0.5**SUB)-1.0)
      COMPAR=DMIN1(9.99D-1,DABS((1.0-(1.0-COLIN)**SUB)*(1.0+P*(1.-COLIN)
     1)))
  190 IF(NTRACE)280,200,200
200   WRITE (6,210)
  210 FORMAT(1X,' ')
C 210 FORMAT(1X,'ENTER SUBROUTINE STEPIT.  COPYRIGHT 1965 J. P. CHANDLER
C    1, INDIANA UNIVERSITY',//,1X,'INITIAL VALUES....'/)
C      WRITE(6,220)(MASK(J),J=1,NV)
C  220 FORMAT(/(I6,6X)/(4X,10I12))
C      WRITE(6,230)(X(J),J=1,NV)
C  230 FORMAT(/F10.4/(10X,10F10.4))
C      WRITE(6,240)(XMAX(J),J=1,NV)
C  240 FORMAT(/F10.4/(10X,10F10.4))
C      WRITE(6,250)(XMIN(J),J=1,NV)
C  250 FORMAT(/F10.4/(10X,10F10.4))
C      WRITE(6,260)(DELTAX(J),J=1,NV)
C  260 FORMAT(/E12.4/(10X,10F10.4))
C      WRITE(6,270)(DELMIN(J),J=1,NV)
C  270 FORMAT(/E15.4/(10X,10F10.4))
  280 CALL OBJFUN
      NF=1
      JOCK=1
  290 IF(NTRACE)320,300,300
C
C XXXMOD JULY 1989
C
  300 I = I
C
C 300 WRITE(6,310)NV,NACTIV,MATRIX,NCOMP,RATIO,ACK,COLIN,COMPAR,CHISQ
C 310 FORMAT(/1X,I3,I3,I4,
C    1I2/1X,F5.1,10X,F5.1,F6.3,10X,
C    2F6.3/1X,F10.8)
  320 IF(NV)2150,2150,330
  330 IF(NTRACE)360,360,340
C
C XXXMOD JULY 1989
C
  340 i = i
c
c 340 WRITE(6,350)
c 350 FORMAT(/10(1X,1H*))
c
  360 DO 370 I=1,NV
      DX(I)=DELTAX(I)
      VEC(I)=0.
      DO 370 J = 1,20
  370 ERR(I,J)=0.
      CHIOLD=CHISQ
      NOSC=0
C
  380 NCIRC=0
      NZIP=0
C
C  MAIN DO LOOP FOR CYCLING THROUGH THE VARIABLES.
C  FIRST TRIAL STEP WITH EACH VARIABLE IS SEPARATE.
  390 NACK=0
      DO 1350 I=1,NV
      OLDVEC(I)=VEC(I)
      VEC(I)=0.0
      TRIAL(I)=0.0
      IF(MASK(I))400,410,400
  400 VEC(I)=-0.0
      NFLAT(I)=1
      GO TO 1350
  410 NACK=NACK+1
      SAVE=X(I)
      IF(SIGNIF*DABS(DX(I))-DABS(X(I))) 580,580,420
  420 X(I)=SAVE+DX(I)
      JVARY=0
      IF(JOCK)440,440,430
  430 JOCK=0
      JVARY=I
  440 NFLAG=1
      IF(X(I)-XMIN(I))460,460,450
  450 IF(X(I)-XMAX(I))470,460,460
  460 NFLAG=NFLAG+3
      GO TO 490
  470 CALL OBJFUN
      IF(ICALLS.LE.IMAX)GO TO 10
      RETURN
   10 CONTINUE
      NF=NF+1
      JVARY=I
      CHIME=CHISQ
      IF(CHISQ-CHIOLD)620,480,490
  480 NFLAG=NFLAG+1
  490 X(I)=SAVE-DX(I)
      IF(X(I)-XMIN(I))590,590,500
  500 IF(X(I)-XMAX(I))510,590,590
  510 CALL OBJFUN
      NF=NF+1
      JVARY=I
      IF(CHISQ-CHIOLD)610,520,530
  520 NFLAG=NFLAG+1
  530 IF(NFLAG-3)540,580,590
  540 IF((CHISQ-CHIME)*(CHIME-2.*CHIOLD+CHISQ))550,590,550
  550 TRIAL(I)=.5*DX(I)*(CHISQ-CHIME)/(CHIME-2.*CHIOLD+CHISQ)
      VEC(I)=TRIAL(I)/DABS(DX(I))
      NFLAT(I)=0
      X(I)=SAVE+TRIAL(I)
      CALL OBJFUN
      NF=NF+1
      IF(CHISQ-CHIOLD)560,570,570
  560 CHIOLD=CHISQ
      JOCK=1
      GO TO 600
  570 TRIAL(I)=0.0
      VEC(I)=0.0
      GO TO 590
  580 VEC(I)=-0.0
      NFLAT(I)=1
  590 X(I)=SAVE
  600 NCIRC=NCIRC+1
      IF(NCIRC-NACTIV)690,1430,1430
  610 DX(I)=-DX(I)
C
C  A LOWER VALUE HAS BEEN FOUND.  HENCE THIS VARIABLE WILL CHANGE.
C
  620 NCIRC=0
      DEL=DX(I)
  630 CHIME=CHIOLD
      CHIOLD=CHISQ
      VEC(I)=VEC(I)+DEL/DABS(DX(I))
      NFLAT(I)=0
      TRIAL(I)=TRIAL(I)+DEL
      DEL=ACK*DEL
      SAVE=X(I)
      X(I)=SAVE+DEL
      IF(X(I)-XMIN(I))680,680,640
  640 IF(X(I)-XMAX(I))650,680,680
  650 CALL OBJFUN
      NF=NF+1
      IF(CHISQ-CHIOLD)630,660,660
660    ZZZ=ACK*CHIME-(ACK+1.0)*CHIOLD+CHISQ
      IF(ZZZ .EQ. 0.) GO TO 661
      CINDER=(0.5/ACK)*(ACK**2*CHIME-(ACK**2-1.0)*CHIOLD-CHISQ)/ZZZ
      GO TO 662
661   CINDER =0.0
662   X(I)=SAVE+CINDER*DEL
      CALL OBJFUN
      NF=NF+1
      IF(CHISQ-CHIOLD)670,680,680
  670 CHIOLD=CHISQ
      TRIAL(I)=TRIAL(I)+CINDER*DEL
      VEC(I)=VEC(I)+CINDER*DEL/DABS(DX(I))
      GO TO 690
  680 X(I)=SAVE
  690 IF(NZIP-1)1340,700,700
  700 IF(DABS(VEC(I))-ACK) 750,710,710
  710 DX(I)=ACK*DABS(DX(I))
      VEC(I)=VEC(I)/ACK
      OLDVEC(I)=OLDVEC(I)/ACK
      DO 720 J=1,MOSQUE
  720 ERR(I,J)=ERR(I,J)/ACK
      IF(NTRACE)750,750,730
  730 WRITE(6,740)I,DX(I)
  740 FORMAT(1X,I3,F10.5)
  750 SUMO=0.0
      SUMV=0.0
      DO 760 J=1,NV
      SUMO=SUMO+OLDVEC(J)**2
  760 SUMV=SUMV+VEC(J)**2
      IF(SUMO*SUMV)1340,1340,770
  770 SUMO=DSQRT(SUMO)
      SUMV=DSQRT(SUMV)
      COSINE=0.0
      DO 780 J=1,NV
  780 COSINE=COSINE+(OLDVEC(J)/SUMO)*(VEC(J)/SUMV)
      IF(NZIP-1)1340,790,800
  790 IF(NACK-NACTIV)1340,820,820
  800 IF(NACK-NACTIV)820,810,810
  810 IF(NZIP-NCOMP)820,830,830
  820 IF(COSINE-COMPAR)1340,830,830
C
C  SIMON SAYS, TAKE AS MANY GIANT STEPS AS POSSIBLE...
C
  830 IF(NTRACE)860,860,840
  840 WRITE(6,850)CHIOLD,(VEC(J),J=1,I)
  850 FORMAT(1X,F10.8,5X,10F9.2/,1X,10F9.2)
  860 NGIANT=0
      NTRY=0
      NRETRY=0
      KL=1
      NOSC=NOSC+1
      IF(NOSC-MOSQUE)890,890,870
  870 NOSC=MOSQUE
      DO 880 K=2,MOSQUE
      CHIOSC(K-1)=CHIOSC(K)
      DO 880 J=1,NV
      XOSC(J,K-1)=XOSC(J,K)
  880 ERR(J,K-1)=ERR(J,K)
  890 DO 900 J=1,NV
      XOSC(J,NOSC)=X(J)
  900 ERR(J,NOSC)=VEC(J)/SUMV
      CHIOSC(NOSC)=CHIOLD
      IF(NOSC-3)960,910,910
C
C  SEARCH FOR A PREVIOUS SUCCESSFUL GIANT STEP IN A DIRECTION MORE
C  NEARLY PARALLEL TO THE DIRECTION OF THE PROPOSED STEP THAN WAS THE
C  IMMEDIATELY PREVIOUS ONE.
C
  910 COXCOM=0.0
      DO 920 J=1,NV
  920 COXCOM=COXCOM+ERR(J,NOSC)*ERR(J,NOSC-1)
      NAH=NOSC-2
  930 NTRY=0
      DO 950 K=KL,NAH
      NRETRY=NAH-K
      COSINE=0.0
      DO 940 J=1,NV
  940 COSINE=COSINE+ERR(J,NOSC)*ERR(J,K)
      IF(COSINE-COXCOM)950,950,970
  950 CONTINUE
  960 CHIBAK=CHI(I)
      GO TO 1020
  970 NTRY=1
      KL=K+1
      IF(NTRACE)1000,1000,980
  980 NT=NOSC-K
      WRITE(6,990)NT
  990 FORMAT(/1X,I2)
 1000 DO 1010 J=1,NV
      SALVO(J)=TRIAL(J)
 1010 TRIAL(J)=(X(J)-XOSC(J,K))/ACK
      CHIBAK=CHIOLD+(CHIOSC(K)-CHIOLD)/ACK
C
 1020 DO 1040 J=1,NV
      XSAVE(J)=X(J)
      TRIAL(J)=ACK*TRIAL(J)
      IF(MASK(J))1040,1030,1040
 1030 TRAAL=X(J)+TRIAL(J)
      TRAIL=DMIN1(TRAAL,XMAX(J))
      X(J)=DMAX1(TRAIL,XMIN(J))
 1040 CONTINUE
      JOCK=0
      JVARY=0
      CALL OBJFUN
      NF=NF+1
      IF(CHISQ-CHIOLD)1050,1080,1080
 1050 CHIBAK=CHIOLD
      CHIOLD=CHISQ
      NGIANT=NGIANT+1
      IF(NTRACE)1020,1020,1060
 1060 WRITE(6,1070)CHISQ,(X(J),J=1,NV)
 1070 FORMAT(1X,F10.8,/(10(1X,F10.5)))
      GO TO 1020
C
 1080 IF(NRETRY)1100,1100,1090
 1090 IF(NGIANT)1150,1150,1100
 1100 CINDER=(0.5/ACK)*(ACK**2*CHIBAK-(ACK**2-1.0)*CHIOLD-CHISQ)/
     1   (ACK*CHIBAK-(ACK+1.0)*CHIOLD+CHISQ)
      DO 1120 J=1,NV
      IF(MASK(J))1120,1110,1120
 1110 CANDER=XSAVE(J)+CINDER*TRIAL(J)
      XIJ=DMIN1(CANDER,XMAX(J))
      X(J)=DMAX1(XIJ,XMIN(J))
 1120 CONTINUE
      JOCK=0
      JVARY=0
      CALL OBJFUN
      NF=NF+1
      IF(CHISQ-CHIOLD)1280,1130,1130
 1130 IF(NGIANT)1170,1140,1170
 1140 IF(NTRY)1150,1170,1150
 1150 DO 1160 J=1,NV
      TRIAL(J)=SALVO(J)
 1160 X(J)=XSAVE(J)
      GO TO 1190
 1170 DO 1180 J=1,NV
      TRIAL(J)=TRIAL(J)/ACK
 1180 X(J)=XSAVE(J)
 1190 IF(NTRACE)1240,1240,1200
 1200 WRITE(6,1210)CHIOLD,NGIANT
 1210 FORMAT(1X,F10.8)
      WRITE(6,1220)(X(J),J=1,NV)
 1220 FORMAT((10(1X,F10.5)))
      WRITE(6,1230)
 1230 FORMAT(/1X,' ')
 1240 IF(NGIANT)1250,1250,1310
 1250 IF(NRETRY)1260,1260,930
 1260 IF(NTRY)1270,1330,1270
 1270 NTRY=0
      GO TO 960
C
 1280 CHIOLD=CHISQ
      JOCK=1
      IF(NTRACE)1310,1310,1290
 1290 STEPS=DFLOAT(NGIANT)+CINDER
      WRITE(6,1300)CHIOLD,STEPS
 1300 FORMAT(1X,F10.8)
      WRITE(6,1220)(X(J),J=1,NV)
      WRITE(6,1230)
1310   IF (NTRY) 1320,380,1320
 1320 NOSC=0
      GO TO 380
1330  NOSC=MAX0(NOSC-1,0)
 1340 CHI(I)=CHIOLD
 1350 CONTINUE
C
C  ANOTHER CYCLE THROUGH THE VARIABLES HAS BEEN COMPLETED.
C  PRINT ANOTHER LINE OF TRACES.
C
      IF(NTRACE)1370,1370,1360
 1360 WRITE(6,850)CHIOLD,(VEC(J),J=1,NV)
1370    CONTINUE
 1380 IF(NZIP)1420,1390,1420
 1390 IF(NTRACE)1420,1420,1400
 1400 WRITE(6,1220)(X(J),J=1,NV)
      WRITE(6,1410)
 1410 FORMAT(1H )
 1420 NZIP=NZIP+1
      GO TO 390
C
C  A MINIMUM HAS BEEN FOUND.  PRINT THE REMAINING TRACES.
C
 1430 IF(NTRACE)1450,1450,1440
 1440 WRITE(6,850)CHIOLD,(VEC(J),J=1,I)
 1450 IF(NTRACE)1470,1470,1460
 1460 WRITE(6,1220)(X(J),J=1,NV)
      WRITE(6,1230)
C
C  DECREASE THE SIZE OF THE STEPS FOR ALL VARIABLES.
C
1470   CONTINUE
 1480 NOSC=0
      NGATE=1
      DO 1520 J=1,NV
      IF(MASK(J))1520,1490,1520
 1490 IF(NFLAT(J))1500,1500,1520
 1500 IF(DABS(DX(J))-DABS(DELMIN(J))) 1520,1520,1510
 1510 NGATE=0
 1520 DX(J)=DX(J)/RATIO
      IF(NGATE)1530,1530,1600
 1530 IF(NTRACE)1570,1570,1540
 1540 WRITE(6,1550)(DX(J),J=1,NV)
 1550 FORMAT(10(1X,1H*)/,(10(1X,F10.5)))
      WRITE(6,1560)
 1560 FORMAT(//)
 1570   GO TO 380
c1580 WRITE(6,1590)(DX(J),J=1,NV)                  
c1590 FORMAT (1X,'OPERAT. TERM.'//10(1X,F10.5))    
C
 1600 CHISQ=CHIOLD
      IF(NTRACE)1630,1610,1610
 1610 WRITE(6,1620)NF
 1620 FORMAT(/1X,I5,23H FUNCTION COMPUTATIONS     )
1630   CONTINUE
1640  IF (IABS(MATRIX-100)-50) 1650,1650,2160
 1650 IF(NACTIV-NV)2160,1660,2160
C
C  COMPUTE THE STANDARD ERRORS AND THE CORRELATIONS.
C
 1660 FAC=RATIO**(MATRIX-100)
      DO 1680 I=1,NV
      DX(I)=DABS(FAC*DX(I))
      XSAVE(I)=X(I)
      JVARY=0
      DO 1670 J=1,2
      X(I)=XSAVE(I)+DX(I)
      CALL OBJFUN
      NF=NF+1
      JVARY=I
      SECOND(1,J)=CHISQ
 1670 DX(I)=-DX(I)
      X(I)=XSAVE(I)
 1680 ERR(I,I)=(SECOND(1,1)-2.0*CHIOLD+SECOND(1,2))/DX(I)**2
      DO 1710 I=2,NV
      IM=I-1
      DO 1710 J=1,IM
      DO 1700 K=1,2
      X(I)=XSAVE(I)+DX(I)
      JVARY=0
      DO 1690 L=1,2
      X(J)=XSAVE(J)+DX(J)
      CALL OBJFUN
      NF=NF+1
      JVARY=J
      SECOND(K,L)=CHISQ
      X(J)=XSAVE(J)
 1690 DX(J)=-DX(J)
      X(I)=XSAVE(I)
 1700 DX(I)=-DX(I)
      ERR(I,J)=0.25*(SECOND(1,1)-SECOND(1,2)-SECOND(2,1)+SECOND(2,2))
     1/DABS(DX(I)*DX(J))
 1710 ERR(J,I)=ERR(I,J)
      IF(NTRACE)1770,1720,1720
 1720 WRITE(6,1730)
 1730 FORMAT(40HSIZES OF INCREMENTS TO BE USED BELOW....)
      WRITE(6,1740)(DX(J),J=1,NV)
 1740 FORMAT(/(10(1X,F10.5)))
      WRITE(6,1750)
 1750 FORMAT(/45H MATRIX OF THE SECOND PARTIAL DERIVATIVES.... /)
      DO 1760 I=1,NV
 1760 WRITE(6,1740)(ERR(I,J),J=1,I)
 1770 DO 1780 I=1,NV
      DO 1780 J=1,I
      IF(ERR(I,J))1780,1790,1780
 1780 CONTINUE
      GO TO 1810
 1790 WRITE(6,1800)
 1800 FORMAT(////123H THE ABOVE MATRIX CONTAINS ONE OR MORE ZEROES.  A L
     1ARGER VALUE OF 'MATRIX' SHOULD BE TRIED, TO SEE IF THEY ARE LEGITI
     2MATE.       )
C
C  INVERT THE MATRIX USING SYMINV2 (COMM. OF THE A.C.M. 6, P. 67).
C
 1810 DET=1.0
      DETLOG=0.
      DO 1820 J=1,NV
 1820 SALVO(J)=1.0
      DO 1970 I=1,NV
      BIGAJJ=0.0
      DO 1850 J=1,NV
      IF(SALVO(J))1830,1850,1830
 1830 IF(DABS(ERR(J,J))-BIGAJJ) 1850,1850,1840
 1840 BIGAJJ=DABS(ERR(J,J))
      K=J
 1850 CONTINUE
      IF(BIGAJJ)1870,1860,1870
 1860 DET=0.0
      GO TO 1980
 1870 SALVO(K)=0.0
c      IF (DET .GT. 1.E48 .OR. DET .LT.1.E-48) GO TO 1871   
       if (det .gt. 1.d48 .or. det .lt.1.d-48) go to 1871   
      GO TO 1875
1871   WRITE (6,1872)
1872   FORMAT (///,'DETERMINANT COMPUTATION WILL CAUSE OVERFLOW.')
      GO TO 2160
1875  DET=DET*ERR(K,K)
      DETLOG=DETLOG+DLOG(DABS(ERR(K,K)))/2.303
      TRIAL(K)=1.0/ERR(K,K)
      ERR(K,K)=0.0
      XSAVE(K)=1.0
      M=K-1
      IF(M)1910,1910,1880
 1880 DO 1900 J=1,M
      XSAVE(J)=ERR(K,J)
      TRIAL(J)=ERR(K,J)*TRIAL(K)
      IF(SALVO(J))1860,1900,1890
 1890 TRIAL(J)=-TRIAL(J)
 1900 ERR(K,J)=0.0
 1910 M=K+1
      IF(M-NV)1920,1920,1960
 1920 DO 1950 J=M,NV
      XSAVE(J)=ERR(J,K)
      IF(SALVO(J))1860,1930,1940
 1930 XSAVE(J)=-XSAVE(J)
 1940 TRIAL(J)=-ERR(J,K)*TRIAL(K)
 1950 ERR(J,K)=0.0
 1960 DO 1970 J=1,NV
      DO 1970 K=J,NV
 1970 ERR(K,J)=ERR(K,J)+XSAVE(J)*TRIAL(K)
      IF(DET)2000,1980,2020
 1980 WRITE(6,1990)
 1990 FORMAT(////67H ERROR MATRIX IS SINGULAR.  'MATRIX' SHOULD PROBABLY
     1 BE INCREASED.              /////)
      GO TO 2150
 2000 WRITE(6,2010)
 2010 FORMAT(////1X,'ERROR MATRIX IS NEGATIVE DEFINITE.  MATRIX SHOULD
     1 PROBABLY BE DECREASED.')
 2020 IF(NTRACE)2050,2030,2030
 2030 WRITE(6,2040)DET,DETLOG
 2040 FORMAT(/1X,F10.5,10X,F10.5)
 2050 DO 2090 I=1,NV
      DO 2060 J=1,I
      ERR(I,J)=ERR(I,J)*2.0
 2060 ERR(J,I)=ERR(I,J)
      IF(ERR(I,I))2070,2070,2090
 2070 WRITE(6,2080)ERR(I,I)
 2080 FORMAT(///50H NEGATIVE OR ZERO MEAN SQUARE ERROR ENCOUNTERED...,
     13X,F10.8/39H 'MATRIX' SHOULD PROBABLY BE DECREASED. ///)
 2090 XSAVE(I)=DSIGN(DSQRT(DABS(ERR(I,I))),ERR(I,I))
      IF(NTRACE)2160,2100,2100
 2100 WRITE(6,2110)
 2110 FORMAT(/////20H STANDARD ERRORS.... )
      WRITE(6,1740)(XSAVE(J),J=1,NV)
      WRITE(6,2120)
 2120 FORMAT(/////45H LOWER TRIANGLE OF THE CORRELATION MATRIX.... /)
      DO 2140 I=2,NV
      IM=I-1
      DO 2130 J=1,IM
 2130 TRIAL(J)=ERR(I,J)/DABS(XSAVE(I)*XSAVE(J))
 2140 WRITE(6,1740)(TRIAL(J),J=1,IM)
 2150 WRITE(6,1620)NF
C
2160  CONTINUE
 2190 JVARY=0
      CALL OBJFUN
      IF(NTRACE)2230,2200,2200
 2200 WRITE(6,2210)(X(J),J=1,NV)
 2210 FORMAT(/1X,/(7(1X,E16.9)))
      WRITE(6,2220)CHISQ
 2220 FORMAT(/1X,F10.8)
 2230 CONTINUE
      RETURN
      END
